@@ -765,10 +765,10 @@ class GatedMemorySelfAttention(layers.Layer):
             shape=(input_shape[-1], dim2),
             initializer="random_normal",
             name="w5",
-            trainable=True,
+            trainable=not self.reshape,
         )
         self.w_input_exp_memory = self.add_weight(
-            shape=(input_shape[-1], dim2),
+            shape=(dim2, input_shape[-1]),
             initializer="zeros",
             name="w6",
             trainable=False,
@@ -785,15 +785,16 @@ class GatedMemorySelfAttention(layers.Layer):
         # ) + tf.math.multiply(inputs, 1.0 - self.retain_factor)
         vk = tf.matmul(inputs, self.w_keys)
         vq = tf.matmul(inputs, self.w_queries)
-        vm = tf.transpose(tf.matmul(inputs, self.w_memory_gate), perm=[0, 2, 1])
-        memory = vm[0, :, :]
-        for i in range(1, inputs.shape.as_list()[0]):
-            memory = memory + vm[i, :, :]
-        self.w_input_exp_memory = tf.math.multiply(
-            self.w_input_exp_memory, self.retain_factor
-        ) + tf.math.multiply(memory, 1.0 - self.retain_factor)
-
-        vv = tf.matmul(inputs, self.w_values) + self.w_input_exp_memory
+        if self.reshape is False:
+            vm = tf.transpose(tf.matmul(inputs, self.w_memory_gate), perm=[0, 2, 1])
+            memory = tf.reduce_mean(vm, axis=0, keepdims=False)
+            self.w_input_exp_memory = tf.math.multiply(
+                self.w_input_exp_memory, self.retain_factor
+            ) + tf.math.multiply(memory, 1.0 - self.retain_factor)
+            vk = tf.math.multiply(vk, self.retain_factor) + tf.math.multiply(
+                self.w_input_exp_memory, 1.0 - self.retain_factor
+            )
+        vv = tf.matmul(inputs, self.w_values)
         kq = tf.matmul(vk, vq, transpose_b=True)
         kqs = kq / self.fact
         if self.norm is not None:
